/*-------------------------------------------------------------------
 * COPYRIGHT (C) 2005 BY SAS INSTITUTE INC.,CARY NC
 *
 * Name: OSE
 * Purpose: Main Entry Point for the OSE engine
 *
 * System Abend Codes:
 * 1 - CMODELS DD Name does not exist or is unuseable
 * 2 - Can not register model.. no model macros to support it
 * 4 - Too many Rule deployment files found
 * 5 - RULES DD name does not exist or is unuseable
 * 6 - Message API Version differs betweeen rule deployment files.
 * 9 - Too many models specified to run.
 *10 - no model metadata found in CMODELS for requested model
 *-------------------------------------------------------------------*/

/*-------------------------------------------------------------------*/
/* The mstored option below directs the macro facility to search for */
/* macros in the BOOTSTRP library.Eventually this library will be    */
/* redefined under the libname COMPILED to be a concatenation of the */
/* MODEL macros and the RULES macros. However at this point the      */
/* complement of models has not been determined so only pick up the  */
/* rules macros                                                      */
libname bootstrp (CRULES);
option mstored sasmstore=bootstrp;

%Utl_ParseCmdLineArgs;

/*-------------------------------------------------------------------*/
/* In Version 2.0 many design time environments can deploy rules to  */
/* be execute by a single OSE server. Each rule deployment creates a */
/* member in the RULES pds. All members are included by the OSE.     */
/* the determination of the PDS member list and the generation of    */
/* %include statement to include all members is done at OSE startup  */
/* by the below macro invocation                                     */
/*                                                                   */
%OSE_Create_OSERules;
%include OSERULES / source2;

/*-------------------------------------------------------------------*/
/* At this point the OSE engine is done reading the contents of the  */
/* rule deployment directory and the _RULES ddname should be freed   */
/* so that SAS no longer has a lock on the PDS which might interfere */
/* with some rules deployment processes                              */
filename _RULES clear;

/*-------------------------------------------------------------------*/
/* Both Models and Rules are expected to change in content during    */
/* the  lifetime of a single isntallation of the software. New       */
/* models are expected to be delivered by the institute 1-2 times a  */
/* a year. Rules are expected to be refined and added to daily by    */
/* the Fraud department. Therefore all code related to these 2       */
/* functions are in %included files that can be regenerated by the   */
/* the mid-tier when changes are made to the models or the rules     */
/*                                                                   */
/* Determine the complement of models in use and generate code to    */
/* support running them.                                             */
/*                                                                   */
%OSE_Create_OSEModels;

/*-------------------------------------------------------------------*/
/* Execute pre-DATA step setup for the engine                        */
/*                                                                   */
%global rul_Abort_Flag rul_Abort_Msg;
%OSE_ENVIRONMENT_INITIALIZATION();


data &rpt_diagnostic_dataset;

/*-------------------------------------------------------------------*/
/* Stop now if there was an error during initializtion               */
/*                                                                   */
if symget("rul_Abort_Flag") not in ("" "0") then
   %UTL_Complain_And_Die(&rul_abort_msg Error during initialization. Stopping.);

/*-------------------------------------------------------------------*/
/* Local Variable Declarations                                       */
/*                                                                   */
Length RPT_verbose 8;    /* Working Variable to control log messages */
Length RPT_rc 8;         /* Working Variable                         */

Length RPT_trx_count 8;  /* Working Variable                         */
                         /* Total # of transaction processed by this */
                         /* datastep since its start                 */

/*-------------------------------------------------------------------*/
/* Configuration                                                     */
/*                                                                   */
RPT_Verbose = &RPT_VERBOSE;
%XMS_Set_Option("VERBOSEVARNAME", "RPT_Verbose");
%XMS_Set_Option("RULEMACROVERSION", "&RUL_MACRO_VERSION");
%XMS_Set_Option("msgAPIVersion", "&RPT_MsgAPIVersion");
%XMS_Set_Option("msgInputEncoding", "&MSG_Input_Encoding");
%XMS_Set_Option("msgOutputEncoding","&MSG_Output_Encoding");

%UTL_LogTime_Begin(TransactionDefs);   
%OSE_Transaction_definitions;
%UTL_LogTime_Finish(TransactionDefs);
   
%OSE_Check_MultiOrgVersion;

/*-------------------------------------------------------------------*/
/*  One Time Initializations                                         */
/*                                                                   */
RPT_trx_count = 0;
      
%UTL_LogTime_Begin(Model_Init);   
%OSE_model_initialization;
%UTL_LogTime_Finish(Model_Init);
   
%UTL_LogTime_Begin(Rule_Init);   
%OSE_Rule_initialization;
%UTL_LogTime_Finish(Rule_Init);   

%UTL_LogTime_Begin(Create_RuleInfo);   
%OSE_Create_RuleInfo;
%UTL_LogTime_Finish(Create_RuleInfo);   

/*-------------------------------------------------------------------*/
/* Await connection from client                                      */
/*                                                                   */
%XMS_Connect_To_Client();

/* Run time error, no retry possible */
if (missing(RPT_rc) or (RPT_rc NE 0)) then do;
   putlog "Unable to connect to XMS subsystem";
   stop;
   end;

   
/*-------------------------------------------------------------------*/
/* Main Transaction Processing Loop...                               */
/*                                                                   */
putlog "Waiting for Message..."; * Do not change (tests use this);


RPT_done = 0;
do while (RPT_done EQ 0);

   /* Recv transaction and unpack data segments                      */
   %XMS_Receive_From_Client();

   /* Intialize Fields used during transaction processing            */
   %OSE_Transaction_Initialization;

   /* OSE Controller Error/Directive                                 */
   /*    0x18 - Directive from Controller to shutdown   (decimal 24) */
   /*    0x20 - Abend in Controller PC routine          (decimal 32) */
   /*    1004 - Controller Unreachable                               */
   if ((RPT_RC EQ 018x) or (RPT_RC EQ 020x) or RPT_RC EQ 1004) then do;
      RPT_done = 1;
      continue;
      end;

   /* Numeric Data Value Error                                       */
   /*                                                                */
   /* Possible RPT_rc (RRR_sys_err_code) values                      */
   /*    1009 - Badly formatted Numeric Data Value in Transaction    */
   /*    1010 - Underflow on reading Data Value from transaction     */
   /*                                                                */
   /*    Variables effected by Bad Data have been set to missing .r  */
   else if ((RPT_rc EQ 1009) or (RPT_rc EQ 1010)) then do;
      %OSE_Warning( RPT_rc );
      end;


   /* Run time error, no retry possible                              */
   /*                                                                */
   /* Possible RPT_rc (RRR_sys_err_code) values                      */
   /*    1000 - No Connection to XMS Server                          */
   /*    1015 - Unrecognized Transaction Segment                     */
   /*    1016 - Unrecognized Transaction                             */
   /*    1017 - Transaction Data Truncated or shorted than expected  */
   /*    1022 - Unrecognized Segment Content Version                 */
   /*                                                                */
   /*                                                                */
   else if (RPT_rc NE 0) then do;
      %OSE_Critical_Error( RPT_rc );
      end;

   /* Run time error, no retry possible (RRR_sys_err_code)           */
   /*                                                                */
   /*    1027 - Mismatch between USC MOHD and deployed rules MO      */
   /*    1036 - Mismatch between USC Build # and rules Build #       */
   /*                                                                */
   %OSE_Check_MultiOrgVersion;
   if (RPT_rc NE 0) then do;
      %OSE_Critical_Error( RPT_rc );
      end;

   /* Received a system command. Either a ping or a change to        */
   /* Verbose setting                                                */
   /*                                                                */
   else if (SMH_tran_type EQ "CMD" ) then do;

      /* Ping Command                                                */
      if (scc_cmd_code EQ 0) then
         scc_cmd_info = "Ready for work - " ||
                     "Version 03.02.00 API " ||
                     substr("&RPT_MsgAPIVersion",2,2) || "." ||
                     substr("&RPT_MsgAPIVersion",4,2) || "." ||
                     substr("&RPT_MsgAPIVersion",6,2);

      /* Set Verbose Level                                           */
      else  if (scc_cmd_code EQ 1) then do;
         if (upcase(scc_cmd_info) =: "VERBOSE=") then do;
            rpt_verbose = input( substr(scc_cmd_info,9,2) , 2.);
            putlog "Verbose Level set to " rpt_verbose;
            end;
         end;

      /* Unknown command                                             */
      else
         scc_cmd_info = "Unknown Command Code...";

      goto RPT_DoneWithTransaction;
      end;

   /* Start of Model related activities. Either Resolving keys or    */
   /* Scoring                                                        */
   %OSE_Set_Timestamp(7);

   /* Received a request to resolve the model lookup keys            */
   /*                                                                */
   /* Possible RRR_sys_err_code values                               */
   /*    1037 - No Resolve macro found for model                     */
   /*                                                                */
   if (smh_resp_req eq '2') then do;
      smh_resp_req = '0';

      %OSE_Model_resolve_keys;

      goto RPT_DoneWithTransaction;
      end;

   /* Successfully received a transaction, up the counter            */
   RPT_trx_count = RPT_trx_count + 1;

   %OSE_Model_Execution;

RPT_DoneWithModels:
      
   %OSE_Set_Timestamp(8);
   %OSE_User_Signature_Validation;
   %OSE_User_Rule_Execution;
   %OSE_Auth_rule_execution;
      
RPT_DoneWithRules:

   /* Save away OSE decision, then do VIP processing                 */
   %OSE_Done_With_Rules;

RPT_DoneWithTransaction:

   /* Send back updated transaction                                  */
   %XMS_Send_To_Client();

end; /* do while (RPT_done EQ 0) */

put "Total # of transactions Processed: " RPT_trx_count;

stop;


/* Miscellany                                                        */
/*                                                                   */
/*   Upon failure to load a lookup list, terminate engine            */
/*                                                                   */
RPT_ListLoadFailure:
RPT_ListLoadFailure_Release:
   putlog "ERROR: Failure loading lookup list.  Aborting...";
   stop; /* abort abend instead? */


/*                                                                   */
/*   Upon failure to create a hash, terminate engine                 */
/*                                                                   */
RPT_REPORT_BAD_HASH_CREATE:
   msg = catx(" ", "&rul_error_create_mark_hash: RC=", rc, msg);
   putlog "ERROR: " msg;
   stop; /* abort abend instead? */


/*                                                                   */
/*   Upon failure to update a hash, just complain and continue       */
/*                                                                   */
RPT_REPORT_BAD_HASH_UPDATE:
   msg = catx(" ", "&rul_error_update_mark_hash: RC=", rc, msg,
                   "RPT_trx_count=", RPT_trx_count
             );
   %OSE_Warning( %scan(%superq(rul_error_update_mark_hash),
                 1,%str( )), msg=msg );
   return;


   
/*                                                                   */
/*   Upon highlight area full, just complain and continue            */
/*                                                                   */
RPT_REPORT_MARK_OVERFLOW:
   msg = catx(" ", "&rul_error_update_mark_hash: ", msg,
                   "RPT_trx_count=", RPT_trx_count
             );
   %OSE_Warning( %scan(%superq(rul_error_update_mark_hash),
                 1,%str( )), msg=msg );
   return;


/*                                                                   */
/* Prevent unref label messages (code is not executed)               */
/*                                                                   */
goto RPT_ListLoadFailure;
goto RPT_ListLoadFailure_Release;
goto RPT_DoneWithRules;
goto RPT_DoneWithModels;
goto RPT_Report_Bad_Hash_Create;
goto RPT_Report_Bad_Hash_Update;
goto RPT_Report_Mark_Overflow;

  
run;
