
CREATE OR REPLACE PROCEDURE RH_DELETE_XACTIONS (
    IN I_TABNAME	CHARACTER(50),
    IN I_COMMIT_POINT	INTEGER,
    IN I_PURGE_DAYS	INTEGER )

  SPECIFIC RH_DELETE_XACTIONS
  DYNAMIC RESULT SETS 1
  LANGUAGE SQL
  NOT DETERMINISTIC
  EXTERNAL ACTION
  MODIFIES SQL DATA
  CALLED ON NULL INPUT
  INHERIT SPECIAL REGISTERS
-----------------------------------------------------------------------------
-- File:  RH_DEL.SQL
--
--
-- Description:
--     Delete transactions from the reporting history database.
--     Commit every i_commit_point rows to control the redo generation.
--
--
--
-- Called By:
--
--
-- Calls:
--     N/A
--
--
-- Notes:
--
--
-- Usage:
--     db2 "call RH_delete_xactions('FRH_AUTH', 10000, 120)"
--
--
-- History:
--     Date          User             Notes
--     2007/02/17    jicimi           created from the SOR's delete_xactions.sql
--     2008/01/14    sinpbh           Removed RUA & RUV references from the proc as R2 is not having these tables.
--     18DEC2008     LEUSHE           S0553598 Removed checking on the SQLSTATE variable. Changed this by setting 
--                                    the v_sqlstate variable SQLSTATE, the conditionally checking v_sqlstate.
--                                    In db2 9.5, accessing the SQLSTATE variable, resets it. Thus, the IF would reset the SQLSTATE 
--                                    value. 
--     2009/04/03    LEUSHE			  Changed the purge criteria to use RHX_DATE_KEY instead of rqo_proc_utc_timestamp, in order to take 
--									  advantage of the index on RHX_DATE_KEY. RHX_DATE_KEY is the date portion of rqo_proc_utc_timestamp
--     2010/08/04    LEUSHE			  Added FRH_NAMP as a table to be valid to run this code on
--     2010/10/05    LEUSHE			  Added FRH_CSAB and FRH_CSAE as a table to be valid to run this code on
--     2010/12/09    LEUSHE			  Remmoved FRH_Vxx_VARAIBLE tables
--     2011/09/01    LEUSHE			  Added the FRH_xxx_VXX tables as valid candidates for purging with this proc
--     2011/11/08    LEUSHE 		  Added FRH_BRST and FRH_BRST_VXX to the valid table list
--
------------------------------------------------------------------------
P1: BEGIN

    DECLARE l_del_count INTEGER DEFAULT 0;
    DECLARE l_rows_deleted INTEGER DEFAULT 0;
    DECLARE l_id BIGINT;

    DECLARE l_delDate       date;
 

    DECLARE l_stmtTemplate varchar(1000);
    DECLARE l_stmtTemplateFact varchar(1000);    

    DECLARE l_delStmt varchar(1000);    

    DECLARE v_sqlstate CHAR(5);		              -- variable for SQLSTATE
    DECLARE v_sqlcode INTEGER;		              -- variable for SQLCODE
    DECLARE v_delete_count INTEGER     DEFAULT 0;     -- variable for deleted row count    
    
    DECLARE SQLCODE INTEGER DEFAULT 0;
    DECLARE SQLSTATE CHAR(5) DEFAULT '00000';
    
    DECLARE l_stmt STATEMENT;    

    DECLARE c_stmt CURSOR WITH HOLD FOR l_stmt;    
    
    DECLARE c_result CURSOR WITH RETURN FOR
       	SELECT i_tabname as TARGET_TAB
       	      ,v_delete_count as DELETE_ROWCOUNT       	      
       	      ,v_sqlstate as OUT_SQLSTATE
       	      ,v_sqlcode as OUT_SQLCODE
    	FROM SYSIBM.SYSDUMMY1;
    
    DECLARE CONTINUE HANDLER FOR SQLEXCEPTION
       SELECT SQLSTATE,
              SQLCODE
       INTO   v_sqlstate,
              v_sqlcode
       FROM SYSIBM.SYSDUMMY1;


    
    /* Initialize variables */
    VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;       


    -- 
    -- default delete criteria 
    --
    set l_delDate = DATE(((current timestamp - current timezone) - i_purge_days days ));



    --
    -- cursor/query to select the main fact table canidate rows for deletion
    --      1.   the RHX_DATE_KEY is older than the threshold (195 days) 
    --           and the transaction has not been identified as confirmed fraud
    --        or
    --      2.   the RHX_DATE_KEY is 365 days old ( and the transaction is fraud ) 
    --
    SET l_stmtTemplate =
      'select cmx_tran_id from '|| rtrim(i_tabname) || ' where ( rhx_date_key <= ? )   FOR UPDATE' ;


    --
    -- cursor(s) to delete the rows returned from the driving cursor 
    --
    SET l_delStmt = 'DELETE FROM '|| rtrim(i_tabname) || ' WHERE CURRENT OF c_stmt ';  


    --
    -- sanity check that only the transaction tables are passed in
    --
       	If (NOT exists (select 1 
				from FRH_TRANSACTION_DEFINITION  
				where rh_Table_name = i_tabname
				      or rh_Table_name = substr(i_tabname,1,8)
					) )
		  THEN
                SET v_sqlcode = -1;
                SET v_sqlstate = '-1';
                SET v_delete_count = 0;             
    	OPEN c_result;
        RETURN -1;
    END IF;
      
    


    SET l_rows_deleted = 0; 
    SET l_del_count = 0;
    PREPARE l_stmt FROM l_stmtTemplate;
    OPEN c_stmt USING l_delDate;

    SET v_sqlstate = SQLSTATE;
    WHILE ( v_sqlstate = '00000' ) DO

       FETCH c_stmt INTO l_id ;
       SET v_sqlstate = SQLSTATE;
       IF ( v_sqlstate = '00000' ) THEN
         EXECUTE IMMEDIATE l_delStmt;

          SET l_del_count = l_del_count + 1;
          IF ( l_del_count = i_commit_point ) THEN
             COMMIT;
             SET l_rows_deleted = l_rows_deleted + l_del_count;
             SET l_del_count = 0;
          END IF;

       END IF;

    END WHILE;

    IF ( l_del_count > 0 ) THEN
       COMMIT;
       SET l_rows_deleted = l_rows_deleted + l_del_count;
       SET v_delete_count = l_rows_deleted;
    END IF;

    CLOSE c_stmt;

    
    VALUES (SQLSTATE, SQLCODE) INTO v_sqlstate, v_sqlcode;
    
    OPEN c_result;

  END P1@

